#ifndef XMEM_CONFIG_H_
#define XMEM_CONFIG_H_

#define XPAR_PS7_DDR_0_BASEADDRESS 0x100000
#define XPAR_PS7_DDR_0_HIGHADDRESS 0x3fffffff
#define XPAR_PS7_QSPI_LINEAR_0_BASEADDRESS 0xfc000000
#define XPAR_PS7_QSPI_LINEAR_0_HIGHADDRESS 0xfdffffff
#define XPAR_PS7_RAM_0_BASEADDRESS 0x0
#define XPAR_PS7_RAM_0_HIGHADDRESS 0x2ffff
#define XPAR_PS7_RAM_1_BASEADDRESS 0xffff0000
#define XPAR_PS7_RAM_1_HIGHADDRESS 0xfffffdff
#define XPAR_AXI_BRAM_0_BASEADDRESS 0x40000000
#define XPAR_AXI_BRAM_0_HIGHADDRESS 0x40001fff

#endif